import PySimpleGUI as sg
from XEdu.hub import Workflow as wf
from PIL import Image
import io
import os
import cv2
import numpy as np

def main():
    # 构建图片默认路径
    default_image_name = 'img.jpg'
    default_image_path = os.path.join(os.getcwd(), default_image_name)

    # 定义主窗口布局
    layout = [
        [sg.Text('智能识图', font=('Arial', 14), justification='center', size=(24, 1))],
        [sg.Text('输入图片路径：'), 
         sg.Input(key='-IMAGE_PATH-', default_text=default_image_path, size=(24, 1))],
        [sg.FileBrowse(button_text="选择图片", file_types=(("所有文件", "*.*"),), target='-IMAGE_PATH-'), 
         sg.Button('确定'), 
         sg.Button('取消')],
        [sg.Button('摄像头采集')]
    ]

    # 创建主窗口
    window = sg.Window('智能识图', layout, size=(240, 320))

    while True:
        event, values = window.read()
        if event in (sg.WINDOW_CLOSED, '取消'):
            break
        elif event == '确定':
            image_path = values['-IMAGE_PATH-'].strip()
            if not image_path:
                sg.popup_error('请输入图片路径！')
                continue

            # 检查文件是否存在
            if not os.path.exists(image_path):
                sg.popup_error(f'文件 {image_path} 不存在！')
                continue

            try:
                # 加载模型
                mm = wf(task='mmedu', checkpoint='cls.onnx')
                # 执行推理
                res, img = mm.inference(data=image_path, img_type='cv2')
                result = mm.format_output(lang="zh")

                # 处理图片显示
                image = Image.open(image_path)
                image.thumbnail((240, 250))
                bio = io.BytesIO()
                image.save(bio, format="PNG")
                img_bytes = bio.getvalue()

                # 构建结果窗口布局
                result_layout = [
                    [sg.Text(f"标签: {result['标签']}", font=('Arial', 12), justification='center')],
                    [sg.Text(f"置信度: {result['置信度']:.2f}%", font=('Arial', 12), justification='center')],
                    [sg.Text(f"预测结果: {result['预测结果']}", font=('Arial', 12), justification='center')],
                    [sg.Image(data=img_bytes)],
                    [sg.Push(), sg.Button('返回', size=(10, 1)), sg.Push()]
                ]

                # 创建结果窗口（非阻塞模式）
                result_window = sg.Window(
                    '推理结果',
                    result_layout,
                    size=(240, 320),
                    finalize=True
                )

                # 处理结果窗口事件
                while True:
                    res_event, res_values = result_window.read()
                    if res_event in (sg.WINDOW_CLOSED, '返回'):
                        result_window.close()
                        break

            except Exception as e:
                sg.popup_error(f'发生错误：\n{str(e)}')

        elif event == '摄像头采集':
            # 摄像头采集图片
            cap = cv2.VideoCapture(0)
            if not cap.isOpened():
                sg.popup_error('无法打开摄像头！')
                continue

            # 创建摄像头采集窗口
            cap_layout = [
                [sg.Button('拍照'), sg.Button('取消')],
                [sg.Image(key='-CAMERA_IMAGE-')]
            ]
            cap_window = sg.Window('摄像头采集', cap_layout, size=(240, 250), finalize=True)

            while True:
                cap_event, cap_values = cap_window.read(timeout=20)
                if cap_event in (sg.WINDOW_CLOSED, '取消'):
                    cap_window.close()
                    cap.release()
                    break
                elif cap_event == '拍照':
                    ret, frame = cap.read()
                    if ret:
                        # 保存图片
                        camera_capture_name = 'camera_capture.jpg'
                        camera_capture_path = os.path.join(os.getcwd(), camera_capture_name)
                        cv2.imwrite(camera_capture_path, frame)
                        cap_window.close()
                        cap.release()
                        # 更新图片路径
                        window['-IMAGE_PATH-'].update(value=camera_capture_path)
                        break

                ret, frame = cap.read()
                if ret:
                    imgbytes = cv2.imencode('.png', frame)[1].tobytes()
                    cap_window['-CAMERA_IMAGE-'].update(data=imgbytes)

    window.close()

if __name__ == '__main__':
    main()